#Enrique Perez
import math

class vec3:
    "A three dimensional vector class."
    def __init__( self ):
        self.x = 0.0
        self.y = 0.0
        self.z = 0.0

    def __repr__(self):
        return '(%s,%s,%s)' % (self.x, self.y, self.z)

# Get the Euclidean distance between this vector and another one.
# @param  another vec3 whose distance to the original will be calculated
    def distance( self, another ):
        return math.sqrt( self.distance2( another ) )

# Get the square of the Euclidean distance between this vector and another one.
# @param  another vec3 whose squared distance to the original will be calculated
    def distance2( self, another ):
        separationX = self.x - another.x
        separationY = self.y - another.y
        separationZ = self.z - another.z
        return separationX * separationX + separationY * separationY + separationZ * separationZ

# Get a complex by removing one axis of this one.
# @param  which - the axis to drop (0=X, 1=Y, 2=Z)
    def dropAxis( self, which ):
        if which == 0:
            return complex( self.y, self.z )
        if which == 1:
            return complex( self.x, self.z )
        if which == 2:
            return complex( self.x, self.y )

# Get a new vec3 identical to another one.
    def getFromVec3( self, another ):
        return self.getFromXYZ( another.x, another.y, another.z )

# Get a new vec3 with the specified x, y, and z components.
    def getFromXYZ( self, x, y, z ):
        self.x = x
        self.y = y
        self.z = z
        return self

# Get the length of the vec3.
    def length( self ):
        return math.sqrt( self.length2() )

# Get the square of the length of the vec3.
    def length2( self ):
        return self.x * self.x + self.y * self.y + self.z * self.z

# Get the difference between the vec3 and another one.
# @param  subtractVec3 vec3 which will be subtracted from the original
    def minus( self, subtractVec3 ):
        return vec3().getFromXYZ( self.x - subtractVec3.x, self.y - subtractVec3.y, self.z - subtractVec3.z )

# Scale each component of this vec3 so that it has a length of 1. If this vec3 has a length of 0, this method has no effect.
    def normalize( self ):
        length = self.length()
        if length == 0.0:
            return
        self.scale( 1.0 / length )

# Get the sum of this vec3 and another one.
# @param  plusVec3 vec3 which will be added to the original
    def plus( self, plusVec3 ):
        return vec3().getFromXYZ( self.x + plusVec3.x, self.y + plusVec3.y, self.z + plusVec3.z )

# Scale each component of this vec3 by a multiplier.
    def scale( self, multiplier ):
        self.x *= multiplier
        self.y *= multiplier
        self.z *= multiplier

# Get a new vec3 by multiplying each component of this one by a multiplier.
    def times( self, multiplier ):
        return vec3().getFromXYZ( self.x * multiplier, self.y * multiplier, self.z * multiplier )

# Get the string representatino of this vec3.
    def toString( self ):
        return 'vec3 xyz: ' + str( self.x ) + ' ' + str( self.y ) + ' ' + str( self.z )
